{% load i18n %}
{% load humanize %}

<div role="tabpanel" class="tab-pane" id="tab_user_mining">
    <p class="report-title">
        {% translate 'Total mining of corporation moons by known mains per month' %}
    </p>
    <div class="table-responsive">
        <table class="table table-striped table-width-fix" id="table_user_mining">
            <thead>
                <tr>
                    <th>{% translate 'Name' %}</th>
                    <th>{% translate 'Corporation' %}</th>
                    <th>{% translate 'State' %}</th>
                    <th>{{ month_minus_3 }}<br>{% translate 'Volume' %}</th>
                    <th class="text-nowrap text-right">{{ month_minus_3 }}<br>{% translate 'Est. Value' %}</th>
                    <th class="text-nowrap text-right">{{ month_minus_2 }}<br>{% translate 'Volume' %}</th>
                    <th class="text-nowrap text-right">{{ month_minus_2 }}<br>{% translate 'Est. Value' %}</th>
                    <th class="text-nowrap text-right">{{ month_minus_1 }}<br>{% translate 'Volume' %}</th>
                    <th class="text-nowrap text-right">{{ month_minus_1 }}<br>{% translate 'Est. Value' %}</th>
                    <th class="text-nowrap text-right">{{ month_current }}<br>{% translate 'Volume' %}</th>
                    <th class="text-nowrap text-right">{{ month_current }}<br>{% translate 'Est. Value' %}</th>
                </tr>
            </thead>
            <tfoot>
                <th class="info">{% translate 'Grand Total' %}</th>
                <th class="info"></th>
                <th class="info"></th>
                <th class="info"></th>
                <th class="info text-right"></th>
                <th class="info text-right"></th>
                <th class="info text-right"></th>
                <th class="info text-right"></th>
                <th class="info text-right"></th>
                <th class="info text-right"></th>
                <th class="info text-right"></th>
            </tfoot>
        </table>
    </div>
    <p class="text-muted">
        {% blocktranslate with last_updated_at=ledger_last_updated|naturaltime|default:"?" %}Last update of mining ledgers at {{ last_updated_at }}{% endblocktranslate %}
    </p>
</div>
