/* dl styling */
dl {
	width: 100%;
	overflow: hidden;
	padding: 0;
  }
  dt {
	float: left;
	width: 30%;
	/* adjust the width; make sure the total of both is 100% */
	padding: 0;
	margin-bottom: 1em;
  }
  dd {
	float: left;
	width: 70%;
	/* adjust the width; make sure the total of both is 100% */
	padding: 0;
	margin-bottom: 1em;
  }

#table_extraction_details thead th:nth-child(4) {
	text-align: right;
}

#table_extraction_details thead th:nth-child(5) {
	text-align: right;
}

#table_extraction_details thead th:nth-child(6) {
	text-align: right;
}

#table_extraction_details tbody td:nth-child(4) {
	text-align: right;
}

#table_extraction_details tbody td:nth-child(5) {
	text-align: right;
}

#table_extraction_details tbody td:nth-child(6) {
	text-align: right;
}

#table_extraction_details tfoot th:nth-child(5) {
	text-align: right;
}

#table_extraction_details tfoot th:nth-child(6) {
	text-align: right;
}
