{% load i18n %}
{% load humanize %}

<div role="tabpanel" class="tab-pane" id="tab_ore_prices">
    <p class="report-title">{% translate 'Current ore prices' %}</p>
    <div class="table-responsive">
        <table class="table table-striped table-width-fix" id="table_ore_prices">
            <thead>
                <tr>
                    <th>{% translate 'Name' %}</th>
                    <th class="text-right">{% translate 'Price (ISK)' %}</th>
                    <th>{% translate 'Group' %}</th>
                    <th>{% translate 'Rarity' %}</th>
                    <th>{% translate 'Description' %}</th>
                </tr>
            </thead>
        </table>
    </div>
    <p class="text-muted">
        {% translate 'Ore prices shown on this report are used for all calculations in this app.' %}
    </p>
    {% if use_reprocess_pricing %}
        <p class="text-muted">
            {% blocktranslate %}Ore prices are calculated as sum of it's reprocessed materials with a yield of {{ reprocessing_yield }}%.{% endblocktranslate %}
        </p>
    {% else %}
        <p class="text-muted">
            {% translate 'The Eve client prices (average prices) are used for ore prices.' %}
        </p>
    {% endif %}
</div>
