{% load i18n %}
{% load humanize %}
{% load moonmining %}

<div role="tabpanel" class="tab-pane" id="tab_{{ category }}">
    <div class="table-responsive">
        <table class="table table-striped table-extractions table-width-fix" id="table_{{ category }}">
            <thead>
                <th>{% translate 'Chunk arrival' %}</th>
                <th>{% translate 'Refinery' %}</th>
                <th>{% translate 'Location' %}</th>
                <th>{% translate 'Labels' %}</th>
                <th class="text-nowrap text-right">{% translate 'Volume (m3)' %}</th>
                <th class="text-nowrap text-right">{% translate 'Est. Value' %}</th>
                <th class="text-nowrap text-right">{% translate 'Est. Mined' %}</th>
                <th></th>
            </thead>
            <tfoot>
                <th>{% translate 'Total' %}</th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th class="text-nowrap text-right"></th>
                <th class="text-nowrap text-right"></th>
                <th class="text-nowrap text-right"></th>
            </tfoot>
        </table>
    </div>
</div>
