{% load i18n %}
{% load humanize %}
{% load moonmining %}

<div class="row">

    <div class="col-md-6">
        {% include 'moonmining/partials/extraction_details_head_base.html' %}
    </div>

    <div class="col-md-6">
        <dl class="dl-horizontal">
            <dt>Chunk arrival:</dt>
            <dd>
                {% if extraction.status == extraction.Status.CANCELED %}<s>{% endif %}
                <span title="{{ extraction.chunk_arrival_at|naturaltime }}">
                    {{ extraction.chunk_arrival_at|datetime }}
                </span>
                {% if extraction.status == extraction.Status.CANCELED %}</s>{% endif %}
            </dd>

            <dt>{% translate 'Total mined volume:' %}</dt>
            <dd>
                {% blocktranslate with volume=total_volume|intword %}{{ volume }} m3{% endblocktranslate %}
            </dd>

            <dt>{% translate 'Total mined value:' %}</dt>
            <dd>
                {% blocktranslate with value=total_value|intword %}{{ value }} ISK{% endblocktranslate %}
            </dd>
        </dl>
    </div>

</div>
