{% load i18n %}

<div role="tabpanel" class="tab-pane show active" id="tab_owned_value">
    <p class="report-title">
        {% translate 'Potential monthly income from owned moons by corporation' %}
    </p>
    <div class="table-responsive">
        <table class="table table-striped table-width-fix" id="table_owned_value">
            <thead>
                <th></th>
                <th>{% translate 'Corporation / Moon' %}</th>
                <th>{% translate 'Region' %}</th>
                <th>{% translate 'Rarity' %}</th>
                <th class="text-nowrap text-right">{% translate 'Est. Value' %}</th>
                <th class="text-nowrap text-right">{% translate '% of Grand Total' %}</th>
                <th class="text-nowrap text-right">{% translate 'Rank of Grand Total' %}</th>
                <th class="text-nowrap text-right">{% translate 'Total' %}</th>
            </thead>
            <tfoot>
                <th></th>
                <th class="info">{% translate 'Grand total' %}</th>
                <th class="info"></th>
                <th class="info"></th>
                <th class="info text-right"></th>
                <th class="info text-right"></th>
                <th class="info text-right"></th>
                <th class="info text-right"></th>
            </tfoot>
        </table>
    </div>
    {% include 'moonmining/partials/value_estimate_legend.html' %}
</div>
